//
//  EpisodeImageView.m
//  MobileDaxhotel
//
//  Created by Raphael Schweikert on 27.01.09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "EpisodeImageView.h"
#import "MobileDaxhotelAppDelegate.h"

#define IMAGE_SWIPE_MOVE_DISTANCE   50.0f
#define TOP_SPACE                   5.0f

@implementation EpisodeImageView


- (id)initWithFrame:(CGRect)frame andAppDelegate:(MobileDaxhotelAppDelegate *)delegate {
    if (self = [super initWithFrame:frame]) {
        self.opaque = YES;
        self.backgroundColor = [UIColor blackColor];
        self.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
        
        isSwiping = NO;
        swipeIsAnimating = NO;
        totalImageCount = 0;
        currentImageIndex = 0;
        
        episodeId = -1;
        
        leftImageView = nil;
        centerImageView = nil;
        rightImageView = nil;
        appDelegate = [delegate retain];
        swipeImages = nil;
    }
    return self;
}


- (void)drawRect:(CGRect)rect {
    // Drawing code
}


- (void)dealloc {
    [appDelegate release];
    if(leftImageView != nil) {
        [leftImageView release];
    }
    if(centerImageView != nil) {
        [centerImageView release];
    }
    if(rightImageView != nil) {
        [rightImageView release];
    }
    if(swipeImages != nil) {
        [swipeImages release];
    }
    [super dealloc];
}

- (UIImage *) imageWithIndex:(NSUInteger) index {
    if(index >= totalImageCount) {
        return nil;
    }
    if(index >= [swipeImages count]) {
        if(index != [swipeImages count]) {
            @throw [[NSException alloc] initWithName:@"Out of bounds" reason:@"Tried to load images in non-sequential order" userInfo:nil];
        }
        [swipeImages addObject:[[appDelegate dataController] image:index forEpisode:episodeId]];
    }
    return [swipeImages objectAtIndex:index];
}

- (void) layoutSubviews {
    if(!swipeIsAnimating && !isSwiping) {
        [self doLayout];
    }
    imageDisplayLabel.frame = CGRectMake(5.0f, 5.0f, self.frame.size.width, 14.0f);
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    if(swipeIsAnimating || isSwiping || [touches count] != 1) {
        return;
    }
    
    lastSwipeStartPositionX = [((UITouch *)[touches anyObject]) locationInView:self].x;
    isSwiping = YES;
    
    leftImageView.hidden = NO;
    centerImageView.hidden = NO;
    rightImageView.hidden = NO;
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
    if(swipeIsAnimating || !isSwiping || [touches count] != 1) {
        return;
    }
    
    CGFloat swipeDistance = [((UITouch *)[touches anyObject]) locationInView:self].x - lastSwipeStartPositionX;
    CGSize contentSize = self.frame.size;
    
    
    leftImageView.frame = CGRectMake(swipeDistance-contentSize.width, TOP_SPACE, contentSize.width, contentSize.height - TOP_SPACE);
    centerImageView.frame = CGRectMake(swipeDistance, TOP_SPACE, contentSize.width, contentSize.height - TOP_SPACE);
    rightImageView.frame = CGRectMake(swipeDistance+contentSize.width, TOP_SPACE, contentSize.width, contentSize.height - TOP_SPACE);
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    if(swipeIsAnimating || !isSwiping) {
        return;
    }
    
    CGFloat swipeDistance = [((UITouch *)[touches anyObject]) locationInView:self].x - lastSwipeStartPositionX;
    NSString *swipeDirection = @"release";
    
    if(currentImageIndex > 0 && swipeDistance > IMAGE_SWIPE_MOVE_DISTANCE) {
        swipeDirection = @"right";
        UIImageView *tempImageView = rightImageView;
        tempImageView.image = nil;
        tempImageView.hidden = YES;
        
        rightImageView = centerImageView;
        centerImageView = leftImageView;
        leftImageView = tempImageView;
        
        currentImageIndex--;
    } else if(currentImageIndex < totalImageCount - 1 && swipeDistance < -IMAGE_SWIPE_MOVE_DISTANCE) {
        swipeDirection = @"left";
        UIImageView *tempImageView = leftImageView;
        tempImageView.image = nil;
        tempImageView.hidden = YES;
        
        leftImageView = centerImageView;
        centerImageView = rightImageView;
        rightImageView = tempImageView;
        
        currentImageIndex++;
    }
    
    [UIView beginAnimations:swipeDirection context:NULL];
    [UIView setAnimationDelegate:self];
    [UIView setAnimationDidStopSelector:@selector(swipeEnded:finished:context:)];
    
    [UIView setAnimationCurve:UIViewAnimationCurveEaseOut];
    [UIView setAnimationDuration:0.3f];
    
    [self doLayout];
    
    swipeIsAnimating = YES;
    [UIView commitAnimations];
    isSwiping = NO;
    [self updateDisplayLabel];
}

- (void)swipeEnded:(NSString *)animationID finished:(BOOL)finished context:(void *)context {
    swipeIsAnimating = NO;
    rightImageView.hidden = YES;
    leftImageView.hidden = YES;
    
    if([animationID isEqualToString:@"right"]) {
        //has more images on the left?
        if(currentImageIndex > 0) {
            leftImageView.image = [self imageWithIndex:currentImageIndex-1];
        }
    } else if([animationID isEqualToString:@"left"]) {
        //has more images on the right?
        if (currentImageIndex < totalImageCount - 1)
        {
            rightImageView.image = [self imageWithIndex:currentImageIndex+1];
        }
    }
}

- (void) doLayout {
    CGSize contentSize = self.frame.size;
    leftImageView.frame = CGRectMake(-contentSize.width, TOP_SPACE, contentSize.width, contentSize.height - TOP_SPACE);
    centerImageView.frame = CGRectMake(0.0f, TOP_SPACE, contentSize.width, contentSize.height - TOP_SPACE);
    rightImageView.frame = CGRectMake(contentSize.width, TOP_SPACE, contentSize.width, contentSize.height - TOP_SPACE);
}

- (void) displayEpisode:(NSUInteger)episode {
    if(episode == episodeId) {
        return;
    }
    swipeImages = (NSMutableArray *)[[appDelegate dataController] imagesForEpisode:episode withStart:0 andLimit:2];
    totalImageCount = [[appDelegate dataController] countImagesForEpisode:episode];
    currentImageIndex = 0;
    if(leftImageView == nil) {
        leftImageView = [self createImageView];
    }
    if(centerImageView == nil) {
        centerImageView = [self createImageView];
    }
    if(rightImageView == nil) {
        rightImageView = [self createImageView];
    }
    [self updateDisplayLabel];
    centerImageView.image = [self imageWithIndex:0];
    centerImageView.hidden = NO;
    rightImageView.image = [self imageWithIndex:1];
    episodeId = episode;
}

- (void) clearEpisode {
    leftImageView.image = nil;
    centerImageView.image = nil;
    rightImageView.image = nil;
    
    episodeId = -1;
    
    [swipeImages release];swipeImages = nil;
}

- (UIImageView *) createImageView {
    UIImageView *result = [[UIImageView alloc] init];
    result.userInteractionEnabled = NO;
    result.opaque = self.opaque;
    result.backgroundColor = self.backgroundColor;
    result.autoresizingMask = self.autoresizingMask;
    result.contentMode = UIViewContentModeCenter;
    result.hidden = YES;
    
    [self addSubview:result];
    
    return result;
}

- (void) updateDisplayLabel {
    if(imageDisplayLabel == nil) {
        imageDisplayLabel = [[UILabel alloc] init];
        imageDisplayLabel.backgroundColor = self.backgroundColor;
        imageDisplayLabel.textColor = [UIColor whiteColor];
        imageDisplayLabel.font = [UIFont fontWithName:@"Helvetica" size:12.0f];
        imageDisplayLabel.opaque = YES;
        [self addSubview:imageDisplayLabel];
    }
    imageDisplayLabel.text = [NSString stringWithFormat:NSLocalizedString(@"imageDisplayLabel", @"%i/%i"), currentImageIndex+1, totalImageCount];
}

@end
